--.
function SRBC:StartGame(timer, player)
	if not SRBC:IsChasers() then return end
	if SRBC.gamestate.started then return end
	if timer == nil then timer = mapheaderinfo[gamemap].srbc_maptimer * TICRATE end
	
	if SRBC:CallHook("ChaseStarting", player) then return end
	
	if P_RandomChance(FU/20) then
		print("\x8F*You feel as though something is missing...") -- teaser
	end
	
	for _, part in ipairs(SRBC.specials.exitparts) do
		P_RemoveMobj(part)
	end
	SRBC.specials.exitparts = {}
	
	if SRBC.specials.exit.valid then
		P_RemoveMobj(SRBC.specials.exit)
	end
	
	SRBC.gameinfo.maxtimer = timer
	SRBC.gamestate.started = SRBC:Timestamp()

	SRBC:UpdateMusic()
	SRBC:RespawnEnemies()
	
	S_StartSound(nil, sfx_supert)
	
	for p in players.iterate do
		if SRBC:IsPlayerMobj(p) then
			
			S_StartSound(mobj, sfx_mixup, p)
			P_FlashPal(p, PAL_MIXUP, 4)
			P_SetOrigin(p.mo, SRBC.finish.x, SRBC.finish.y, SRBC.finish.z)
			P_InstaThrust(p.mo, SRBC.finish.angle, p.speed)
			p.powers[pw_invulnerability] = TICRATE*2
			p.mo.angle = SRBC.finish.angle
			if SRBC.finish.flip then p.mo.flags2 = $|MF2_OBJECTFLIP 
			else p.mo.flags2 = $ & ~MF2_OBJECTFLIP end
			SRBC:ReviveGhost(p)
			
		end
	end
	
	SRBC:CallHook("ChaseStarted", player)
	
end


--.
function SRBC:StartOvertime()
	if not SRBC:IsChasers() or not SRBC.gamestate.started then return end
	if SRBC.gamestate.overtime then return end
	
	if SRBC:CallHook("OvertimeStarting") then return end
	
	SRBC.gamestate.overtime = SRBC:Timestamp()
	SRBC:UpdateMusic()
	SRBC:RespawnEnemies()
	
	S_StartSound(nil, P_RandomRange(41,43))
	
	for p in players.iterate do
		if p.valid then
			if p.srbc.ghost then
				SRBC:PlayerFinish(p, true)
			else
				p.srbc.overtime = min(max(p.srbc.timeaverage*2, TICRATE), SRBC.gameinfo.maxtimer)
			end
			P_FlashPal(p, PAL_MIXUP, 4)
		end
	end
	
	SRBC:CallHook("OvertimeStarted")
end

--.
function SRBC:Lap(p)
	if not SRBC:IsChasers() then return end
	if not SRBC.gamestate.started then return end
	if p.srbc.over then return end
	
	if SRBC:CallHook("PlayerLap", p) then return end
	
	P_SpawnMobjFromMobj(p.mo, 0, 0, p.mo.height/2, MT_EXITFLASH)
	
	P_SetOrigin(p.mo, SRBC.finish.x, SRBC.finish.y, SRBC.finish.z)
	P_InstaThrust(p.mo, SRBC.finish.angle, p.speed)
	p.mo.angle = SRBC.finish.angle
	if SRBC.finish.flip then p.mo.flags2 = $|MF2_OBJECTFLIP 
	else p.mo.flags2 = $ & ~MF2_OBJECTFLIP end
	
	P_SpawnMobjFromMobj(p.mo, 0, 0, p.mo.height/2, MT_EXITFLASH)
	
	local last = SRBC.gamestate.started
	for _, laptime in ipairs(p.srbc.laptimes) do
		last = $ + laptime
	end
	table.insert(p.srbc.laptimes, SRBC:TimeSince(last))
	p.srbc.laps = $+1
	
	p.srbc.timeaverage = 0
	for _, laptime in ipairs(p.srbc.laptimes) do
		p.srbc.timeaverage = $ + laptime
	end
	p.srbc.timeaverage = $/p.srbc.laps
	
	S_StartSound(p.mo, sfx_mixup)
	P_FlashPal(p, PAL_MIXUP, 4)
	
	if SRBC.gamestate.overtime then
		local timeback = p.srbc.timeaverage/5*2
		
		if mapheaderinfo[gamemap].srbc_partime then
			timeback = min(p.srbc.timeaverage, mapheaderinfo[gamemap].srbc_partime*TICRATE)/5*2
		end
		
		p.srbc.overtime = min($ + timeback, SRBC:SinceOvertime() + SRBC.gameinfo.maxtimer)
		chatprint("\130*"..p.name.." cleared lap "..p.srbc.laps.." getting "..SRBC:TicsToTime(timeback).." back on the clock!", true)
	else
		chatprint("\130*"..p.name.." cleared lap "..p.srbc.laps.." with a time of "..SRBC:TicsToTime(p.srbc.laptimes[p.srbc.laps], true).."!", true)
	end
end



--> Game Thinker.
addHook("ThinkFrame", function()
	if not SRBC:IsChasers() then return end
	
	SRBC.gameinfo.playercount = 0
	SRBC.gameinfo.playersleft = 0
	SRBC.gameinfo.ghosts = 0
		
	for p in players.iterate do
		if p.valid then SRBC.gameinfo.playercount = $+1 end
		
		if SRBC:IsPlayerMobj(p) then
			if not ((p.srbc.ghost and p.srbc.ghost.timer == 0) or p.srbc.over) then SRBC.gameinfo.playersleft = $+1 end
			if (p.srbc.ghost and p.srbc.ghost.timer == 0) and not p.srbc.over then SRBC.gameinfo.ghosts = $+1 end
		end
	end
	
	if SRBC.gamestate.started then
		
		if SRBC:TimeSince(SRBC.gamestate.started) >= SRBC.gameinfo.maxtimer and not SRBC.gamestate.overtime then
			SRBC:StartOvertime()
		end
		
		if SRBC.gamestate.overtime and SRBC.gameinfo.playersleft == 2 and SRBC.gameinfo.ghosts == 0 and not SRBC.gamestate.faceoff then
			SRBC.gamestate.faceoff = SRBC:Timestamp()
			
			-- next time.
		end
		
		if SRBC.gameinfo.playersleft == 0 and not SRBC.gamestate.gameover then SRBC.gamestate.gameover = SRBC:Timestamp() end
		
	end
end)


--> Player Game Thinker.
addHook("PlayerThink", function(p)
	if not SRBC:IsChasers() then return end
	if p.valid then
		if SRBC.gamestate.started and not p.srbc.over then
			if SRBC.gamestate.overtime and SRBC:SinceOvertime() == p.srbc.overtime then
				SRBC:PlayerFinish(p, true)
			end
			
			if SRBC:IsPlayerMobj(p) and p.srbc.nerfed == 0 and SRBC.gameinfo.playersleft == 1 and SRBC.gameinfo.ghosts == 0 and SRBC.gameinfo.playercount > 2 then
				p.srbc.nerfed = 1
			end
			
			if SRBC.gamestate.gameover then SRBC:PlayerFinish(p, true) end
		end
	end
end)


--> Reset the game state on map change.
addHook("MapChange", function()
	if not SRBC:IsChasers() then return end
	
	SRBC.gamestate.started = 0
	SRBC.gamestate.faceoff = 0
	SRBC.gamestate.overtime = 0
	SRBC.gamestate.timer = 0
	SRBC.gamestate.gameover = 0
	
	for p in players.iterate do
		if p.valid then
			
			p.srbc.laps = 0
			p.srbc.laptimes = {}
			p.srbc.timeaverage = 0
			
		end
	end
end)